# -*- coding: utf-8 -*-
# Copyright (C) 2019 fjord-technologies
# SPDX-License-Identifier: GPL-3.0-or-later
"""xtrargon_ng.modules.server"""

import logging
import os
import netifaces

from dwho.classes.modules import DWhoModuleBase, MODULES
from httpdis.ext.httpdis_json import HttpReqErrJson
from sonicprobe import helpers
from sonicprobe.libs.moresynchro import RWLock


DRBD_STATE_FILE = "/proc/drbd"

LOG             = logging.getLogger('xtrargon_ng.modules.server')


class ServerModule(DWhoModuleBase):
    MODULE_NAME = 'server'

    LOCK = RWLock()

    # pylint: disable-msg=attribute-defined-outside-init
    def safe_init(self, options):
        self.lock_timeout = self.config['general']['lock_timeout']

    def is_primary(self, request):
        if not self.LOCK.acquire_read(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for reading after %s seconds" % self.lock_timeout)

        try:
            if not helpers.has_len(self.config['general'].get('primary_ifname')):
                raise HttpReqErrJson(500, "missing primary_ifname option in section general")
            return self.config['general']['primary_ifname'] in netifaces.interfaces() # pylint: disable-msg=c-extension-no-member
        except HttpReqErrJson:
            raise
        except Exception as e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()

    def has_drbd(self, request):
        if not self.LOCK.acquire_read(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for reading after %s seconds" % self.lock_timeout)

        try:
            return os.path.isfile(DRBD_STATE_FILE)
        except HttpReqErrJson:
            raise
        except Exception as e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()


if __name__ != "__main__":
    def _start():
        MODULES.register(ServerModule())
    _start()
